@@ -1,10 +1,11 @@
 use clap::ArgMatches;
 use clap_complete::Shell::*;
+use clap_complete_nushell::Nushell;
 use regex::Regex;
 use rusqlite::Error;
 
 use crate::config::ConfigOptions;
-use crate::graph::zk_graph_dot_output;
+use crate::graph::{vizk, zk_graph_dot_output, zk_graph_json_output};
 use crate::zettel::strip_multiple_whitespace;
 use crate::Database;
 use crate::Zettel;
@@ -67,7 +68,10 @@ impl Printer
         let mut empty_diff = vec!["".to_string(); len_diff];
         self.additional.append(&mut empty_diff);
 
-        for (z, a) in self.zettel.iter().zip(&self.additional) {
+        let mut zip = self.zettel.iter().zip(&self.additional).collect::<Vec<_>>();
+        zip.sort_by(|a, b| a.partial_cmp(b).unwrap());
+
+        for (z, a) in zip {
             let mut result = self.format.to_string();
 
             result = result.replace("%t", &z.title);
@@ -86,10 +90,12 @@ impl Default for Printer
 {
     fn default() -> Printer
     {
-        Printer { zettel: vec![],
-                  additional: vec![],
-                  format: "[%p] %t".to_string(),
-                  link_separator: "|".to_string() }
+        Printer {
+            zettel: vec![],
+            additional: vec![],
+            format: "[%p] %t".to_string(),
+            link_separator: "|".to_string(),
+        }
     }
 }
 
@@ -98,10 +104,10 @@ pub fn sync(matches: &ArgMatches, cfg: &ConfigOptions) -> Result<(), Error>
     Database::new(&cfg.db_file())?.init()?;
 
     let project = realproject(if let Some(p) = matches.get_one::<String>("PROJECT") {
-                                  p
-                              } else {
-                                  ""
-                              });
+        p
+    } else {
+        ""
+    });
     if let Some(title) = matches.get_one::<String>("CREATE") {
         create(cfg, title, project)?;
     } else if let Some(path) = matches.get_one::<String>("UPDATE") {
@@ -109,11 +115,12 @@ pub fn sync(matches: &ArgMatches, cfg: &ConfigOptions) -> Result<(), Error>
     } else if let Some(title) = matches.get_one::<String>("MOVE") {
         mv(cfg, title, project)?;
     } else if matches.contains_id("RENAME") {
-        let args_arr = matches.get_many::<String>("RENAME")
-                              .unwrap_or_default()
-                              .map(|a| a.to_string())
-                              .collect::<Vec<_>>();
-        rename(cfg, &args_arr)?;
+        let args = matches
+            .get_many::<String>("RENAME")
+            .unwrap_or_default()
+            .map(|a| a.to_string())
+            .collect::<Vec<String>>();
+        rename(cfg, &args[0], &args[1])?;
     } else if matches.get_flag("GENERATE") {
         generate(cfg)?;
     }
@@ -163,8 +170,18 @@ pub fn query(matches: &ArgMatches, cfg: &ConfigOptions) -> Result<(), Error>
         zs = filter_isolated(zs);
     }
 
-    if matches.get_flag("GRAPH") {
-        zk_graph_dot_output(&zs);
+    if let Some(graph) = matches.get_one::<String>("GRAPH") {
+        match graph.as_str() {
+            "vizk" => vizk(&zs),
+            "dot" => zk_graph_dot_output(&zs),
+            "json" => zk_graph_json_output(&zs),
+            _ => {
+                eprintln!(
+                    "error: expected one of 'json', 'dot', 'vizk' (got: '{}')",
+                    graph
+                );
+            }
+        }
         return Ok(());
     }
 
@@ -204,8 +221,10 @@ pub fn ls(matches: &ArgMatches, cfg: &ConfigOptions) -> Result<(), Error>
         "ghosts" => print_list_of_strings(&db.zettel_not_yet_created()?),
         "projects" => print_list_of_strings(&db.list_projects()?),
         "path" => println!("{}", cfg.zettelkasten),
-        _ => eprintln!("error: expected one of: 'tags', 'ghosts', 'projects', 'path'; got '{}'",
-                       obj),
+        _ => eprintln!(
+            "error: expected one of: 'tags', 'ghosts', 'projects', 'path'; got '{}'",
+            obj
+        ),
     }
     Ok(())
 }
@@ -220,11 +239,19 @@ pub fn compl(matches: &ArgMatches) -> Result<(), Error>
         "bash" => Some(Bash),
         "fish" => Some(Fish),
         _ => None,
+        // Note that Nushell is handled separately since the Enum types don't match
     };
 
+    let app = &mut cli::build();
     if let Some(sh) = sh {
-        let app = &mut cli::build();
         clap_complete::generate(sh, app, app.get_name().to_string(), &mut std::io::stdout());
+    } else if shell.as_str() == "nu" || shell.as_str() == "nushell" {
+        clap_complete::generate(
+            Nushell,
+            app,
+            app.get_name().to_string(),
+            &mut std::io::stdout(),
+        );
     } else {
         eprintln!("error: '{}' isn't a (supported) shell", shell);
     }
@@ -235,9 +262,11 @@ pub fn compl(matches: &ArgMatches) -> Result<(), Error>
 /// Print every element in the list of Strings on an individual line
 fn print_list_of_strings(elems: &[String])
 {
-    elems.iter().for_each(|e| {
-                    println!("{}", e);
-                })
+    let mut sorted = elems.to_vec();
+    sorted.sort();
+    sorted.iter().for_each(|e| {
+        println!("{}", e);
+    })
 }
 
 /// Keep only those Zettel whose title matches the provided regex
@@ -245,14 +274,14 @@ fn filter_title(zs: Vec<Zettel>, pattern: &str, exact: bool) -> Vec<Zettel>
 {
     let re = Regex::new(&format!("^{}$", pattern)).unwrap();
     zs.into_iter()
-      .filter(|z| {
-          if exact {
-              pattern == z.title
-          } else {
-              re.is_match(&z.title)
-          }
-      })
-      .collect()
+        .filter(|z| {
+            if exact {
+                pattern == z.title
+            } else {
+                re.is_match(&z.title)
+            }
+        })
+        .collect()
 }
 
 /// Keep only those Zettel whose project matches the provided regex
@@ -260,23 +289,23 @@ fn filter_project(zs: Vec<Zettel>, pattern: &str, exact: bool) -> Vec<Zettel>
 {
     let re = Regex::new(&format!("^{}$", pattern)).unwrap();
     zs.into_iter()
-      .filter(|z| {
-          if exact {
-              pattern == z.project
-          } else {
-              re.is_match(&z.project)
-          }
-      })
-      .collect()
+        .filter(|z| {
+            if exact {
+                pattern == z.project
+            } else {
+                re.is_match(&z.project)
+            }
+        })
+        .collect()
 }
 
 /// Keep only those Zettel that contain the pattern in their text
 fn filter_text(zs: Vec<Zettel>, pattern: &str, cfg: &ConfigOptions) -> Vec<(Zettel, String)>
 {
     zs.into_iter()
-      .map(|z| (z.clone(), z.find_pattern(cfg, pattern)))
-      .filter(|(_, t)| !t.is_empty())
-      .collect()
+        .map(|z| (z.clone(), z.find_pattern(cfg, pattern)))
+        .filter(|(_, t)| !t.is_empty())
+        .collect()
 }
 
 /// Keep only those Zettel that have at least one tag (or subtag) that matches the regex
@@ -284,31 +313,33 @@ fn filter_tag(zs: Vec<Zettel>, pattern: &str, exact: bool) -> Vec<Zettel>
 {
     let re = Regex::new(&format!("^{}(/.*)?$", pattern)).unwrap();
     zs.into_iter()
-      .filter(|z| {
-          if let Some(t) = z.tags.first() {
-              return if exact { pattern == t } else { re.is_match(t) };
-          }
-          false
-      })
-      .collect()
+        .filter(|z| {
+            for tag in &z.tags {
+                if pattern == tag || (!exact && re.is_match(tag)) {
+                    return true;
+                };
+            }
+            false
+        })
+        .collect()
 }
 
 /// Keep only those Zettel that neither link to other notes, nor have links pointing to them
 fn filter_isolated(zs: Vec<Zettel>) -> Vec<Zettel>
 {
     zs.into_iter()
-      .filter(|z| z.links.is_empty() && z.backlinks.is_empty())
-      .collect()
+        .filter(|z| z.links.is_empty() && z.backlinks.is_empty())
+        .collect()
 }
 
 /// Keep only the Zettel that are both in A and B
 fn intersect<T: Eq + Clone>(a: &[T], b: &[T]) -> Vec<T>
 {
     a.to_owned()
-     .iter()
-     .cloned()
-     .filter(|z| b.contains(z))
-     .collect::<Vec<_>>()
+        .iter()
+        .cloned()
+        .filter(|z| b.contains(z))
+        .collect::<Vec<_>>()
 }
 
 /// Return all the Zettel that are linked to by the pattern-matched zettel
@@ -318,16 +349,16 @@ fn fwlinks(all: &[Zettel], linked_from: &str, exact: bool) -> Vec<Zettel>
     // them
     let fwlinks = &filter_title(all.to_owned(), linked_from, exact);
     all.iter()
-       .cloned()
-       .filter(|z| {
-           for fw in fwlinks {
-               if fw.links.contains(&z.title) {
-                   return true;
-               }
-           }
-           false
-       })
-       .collect()
+        .cloned()
+        .filter(|z| {
+            for fw in fwlinks {
+                if fw.links.contains(&z.title) {
+                    return true;
+                }
+            }
+            false
+        })
+        .collect()
 }
 
 /// Return all the Zettel that link to the given title/pattern, within the provided list of Zettel
@@ -335,16 +366,16 @@ fn backlinks(all: &[Zettel], links_to: &str, exact: bool) -> Vec<Zettel>
 {
     let re = Regex::new(&format!("^{}$", links_to)).unwrap();
     all.iter()
-       .cloned()
-       .filter(|z| {
-           for l in z.links.clone() {
-               if links_to == l || (!exact && re.is_match(&l)) {
-                   return true;
-               };
-           }
-           false
-       })
-       .collect()
+        .cloned()
+        .filter(|z| {
+            for l in z.links.clone() {
+                if links_to == l || (!exact && re.is_match(&l)) {
+                    return true;
+                };
+            }
+            false
+        })
+        .collect()
 }
 
 /// Based on the CLI arguments and the config options, *maybe* add a new entry to the database
@@ -392,32 +423,19 @@ fn create(cfg: &ConfigOptions, title: &str, project: &str) -> Result<(), Error>
 }
 
 /// Rename a note, but keep it in the same project
-fn rename(cfg: &ConfigOptions, arr: &[String]) -> Result<(), Error>
+fn rename(cfg: &ConfigOptions, old_title: &str, new_title: &str) -> Result<(), Error>
 {
     let db = Database::new(&cfg.db_file())?;
 
-    // basically, look thru the values provided by clap, and extract the first Zettel title that
-    // exists and is different from the new title
-    let old_title: String = arr.iter()
-                               .cloned()
-                               .find(|x| !db.find_by_title(x).unwrap_or_default().is_empty())
-                               .unwrap_or(String::from(""));
-    let new_title: String = arr.last().unwrap_or(&String::from("")).to_string();
-
     if old_title == new_title {
-        eprintln!("error: first match is the same as the new title ('{}'), so no rename",
-                  old_title);
+        eprintln!(
+            "error: old title is the same as the new title ('{}'), so no rename",
+            old_title
+        );
         return Ok(());
     }
 
-    let results = db.find_by_title(&old_title)?;
-
-    // check if there's already a note with this title
-    let overwrite_failsafe = db.find_by_title(&new_title)?;
-    if overwrite_failsafe.first().is_some() {
-        eprintln!("error: a note with the new title already exists: won't overwrite");
-        return Ok(());
-    }
+    let results = db.find_by_title(old_title)?;
 
     let old_zettel = if results.first().is_none() {
         eprintln!("error: no Zettel with that title");
@@ -425,7 +443,20 @@ fn rename(cfg: &ConfigOptions, arr: &[String]) -> Result<(), Error>
     } else {
         results.first().unwrap()
     };
-    let new_zettel = Zettel::new(&new_title, &old_zettel.project);
+    let new_zettel = Zettel::new(new_title, &old_zettel.project);
+
+    if !file_exists(&old_zettel.filename(cfg)) {
+        eprintln!("error: the Zettel does not exist on the filesystem");
+        db.delete(&old_zettel)?;
+        return Ok(());
+    }
+
+    let exists_in_db = db.find_by_title(new_title)?.first().is_some();
+    let exists_in_fs = file_exists(&new_zettel.filename(cfg));
+    if exists_in_db || exists_in_fs {
+        eprintln!("error: a note with the new title already exists: won't overwrite");
+        return Ok(());
+    }
 
     let mut dial = dialoguer::Confirm::new();
     let prompt = dial.with_prompt(format!("{} --> {}", old_title, new_title));
@@ -433,20 +464,18 @@ fn rename(cfg: &ConfigOptions, arr: &[String]) -> Result<(), Error>
     // If the user confirms, change the note's title, and update the links to this Zettel
     if prompt.interact().unwrap_or_default() {
         crate::io::rename(&old_zettel.filename(cfg), &new_zettel.filename(cfg));
-        db.change_title(old_zettel, &new_title).unwrap();
+        db.change_title(old_zettel, new_title).unwrap();
         // It's not enough that we renamed the file. We need to update all references to it!
-        let backlinks = backlinks(&db.all()?, &old_title, true);
+        let backlinks = backlinks(&db.all()?, old_title, true);
         backlinks.iter().for_each(|bl| {
-                            let contents = crate::io::file_to_string(&bl.filename(cfg));
-                            // The link might span over multiple lines. We must account for that
-                            let regex_string =
-                                &format!(r"\[\[{}\]\]", old_title).replace(' ', r"[\n\t ]");
-                            let old_title_reg = Regex::new(regex_string).unwrap();
-                            let new_contents =
-                                old_title_reg.replace_all(&contents, format!(r"[[{}]]", new_title));
-                            crate::io::write_to_file(&bl.filename(cfg), &new_contents);
-                            db.update(cfg, bl).unwrap();
-                        })
+            let contents = crate::io::file_to_string(&bl.filename(cfg));
+            // The link might span over multiple lines. We must account for that
+            let regex_string = &format!(r"\[\[{}\]\]", old_title).replace(' ', r"[\n\t ]");
+            let old_title_reg = Regex::new(regex_string).unwrap();
+            let new_contents = old_title_reg.replace_all(&contents, format!(r"[[{}]]", new_title));
+            crate::io::write_to_file(&bl.filename(cfg), &new_contents);
+            db.update(cfg, bl).unwrap();
+        })
     }
 
     Ok(())
@@ -458,37 +487,41 @@ fn mv(cfg: &ConfigOptions, pattern: &str, project: &str) -> Result<(), Error>
     let db = Database::new(&cfg.db_file())?;
 
     let re = Regex::new(pattern).unwrap();
-    let zs: Vec<Zettel> = db.all()?
-                            .into_iter()
-                            .filter(|z| re.is_match(&z.title))
-                            .collect();
+    let zs: Vec<Zettel> = db
+        .all()?
+        .into_iter()
+        .filter(|z| re.is_match(&z.title))
+        .collect();
 
     let mut printer = Printer::default();
     printer.set_zettelkasten(zs.clone());
     printer.print(cfg);
 
     let mut dial = dialoguer::Confirm::new();
-    let prompt =
-        dial.with_prompt(format!(">> These notes will be transferred to the {}. Proceed?",
-                                 if project.is_empty() {
-                                     "main zettelkasten".to_string()
-                                 } else {
-                                     format!("'{}' project", project)
-                                 }));
+    let prompt = dial.with_prompt(format!(
+        ">> These notes will be transferred to the {}. Proceed?",
+        if project.is_empty() {
+            "main zettelkasten".to_string()
+        } else {
+            format!("'{}' project", project)
+        }
+    ));
 
     // If the user confirms, change the notes' projects, both the system path and in database
     if prompt.interact().unwrap_or_default() {
         crate::io::mkdir(&format!("{}/{}", cfg.zettelkasten, project));
-        let new_notes = zs.iter().map(|z| Zettel { title: z.title.clone(),
-                                                   project: project.to_string(),
-                                                   links: z.links.clone(),
-                                                   backlinks: z.backlinks.clone(),
-                                                   tags: z.tags.clone() });
+        let new_notes = zs.iter().map(|z| Zettel {
+            title: z.title.clone(),
+            project: project.to_string(),
+            links: z.links.clone(),
+            backlinks: z.backlinks.clone(),
+            tags: z.tags.clone(),
+        });
         let pairs = zs.iter().zip(new_notes);
         pairs.for_each(|(old, new)| {
-                 crate::io::rename(&old.filename(cfg), &new.filename(cfg));
-                 db.change_project(old, project).unwrap();
-             });
+            crate::io::rename(&old.filename(cfg), &new.filename(cfg));
+            db.change_project(old, project).unwrap();
+        });
     }
 
     Ok(())
@@ -525,8 +558,10 @@ fn generate(cfg: &ConfigOptions) -> Result<(), Error>
     mem_db.generate(cfg)?;
     mem_db.write_to(&cfg.db_file())?;
 
-    println!("database generated successfully, took {}ms",
-             start.elapsed().as_millis());
+    println!(
+        "database generated successfully, took {}ms",
+        start.elapsed().as_millis()
+    );
 
     Ok(())
 }
